#include "info.h"
#include "player.h"
#include "game_display.h"
#include "icons.h"
#include <string.h>

#ifndef DEBUG
WINDOW *info_win=NULL;

void info_win_create()
{
    info_win=newwin(ICON_H*GAREA_H, INFO_WIN_W, 0, ICON_W*GAREA_W);
    scrollok(info_win,FALSE);
    keypad(info_win,TRUE);
}

void info_win_destroy()
{
    delwin(info_win);
}

void info_win_update_player_icon()
{
    win_icon_dump(info_win, 2, 2, ICON_PLAYER0+player.player);
}

bool map_overwrite_bad(xy_t x, xy_t y)
{
    xy_t xx=(x%2)?x-1:x;
    xy_t yy=(y%2)?y-1:y;
    if(x%2){
        if(map->buf[y][xx]==ICON_MASK)
            return TRUE;
    }
    else if(x<MAP_W-1){
        if(map->buf[y][x+1]==ICON_MASK)
            return TRUE;
    }
    if(y%2){
        if(map->buf[yy][x]==ICON_MASK)
            return TRUE;
    }
    else if(y<MAP_H-1){
        if(map->buf[y+1][x]==ICON_MASK)
            return TRUE;
    }
    if(x%2&&y%2){
        if(map->buf[yy][xx]==ICON_MASK)
            return TRUE;
    }
    else if(x<MAP_W-1&&y<MAP_H-1){
        if(map->buf[y+1][x+1]==ICON_MASK)
            return TRUE;
    }
    return FALSE;
}

void info_win_update_map(su_t have_map)
{
    su_t i;
    su_t mb=1;
    for(i=0;i<4;i++,mb*=2){
        if(mb&have_map)
            info_win_dump_map(i);
    }
}

void info_win_dump_map(su_t mappc)
{
    xy_t x,xx,y,yy;
    su_t icon;
    switch(mappc){
        case 0:     x=HMAP_W;   y=0;        break;
        case 1:     x=HMAP_W;   y=HMAP_H;   break;
        case 2:     x=0;        y=HMAP_H;   break;
        case 3:     x=0;        y=0;        break;
    }
    for(xx=x;xx<x+HMAP_W;xx++){
        for(yy=y;yy<y+HMAP_H;yy++){
            switch((icon=map->buf[yy][xx])){
                case ICON_WALL:
                    if(map_overwrite_bad(xx,yy))
                        break;
                    wattrset(info_win,COLOR_PAIR(ICON_XXX+1));
                    mvwaddch(info_win,6+yy/2,4+xx/2,icon_to_mapchar(icon));
                    break;
                case ICON_MASK:
                case ICON_EXIT:
                    wattrset(info_win,COLOR_PAIR(icon));
                    mvwaddch(info_win,6+yy/2,4+xx/2,icon_to_mapchar(icon));
                    break;
                default:
                    break;
            }
        }
    }
}

void info_win_map_erase_mask(xy_t x, xy_t y)
{
    if(map_overwrite_bad(x,y))
        return;
    wattrset(info_win,COLOR_PAIR(0));
    mvwaddch(info_win,5+y/2,4+x/2,icon_to_mapchar(ICON_SPACE));
}

#endif

void info_win_display()
{
    #ifndef DEBUG
    wattrset(info_win,0);
    mvwprintw(info_win, 0,(INFO_WIN_W-(strlen(map->name)+2))/2, " %s ",map->name);
    mvwprintw(info_win, 2,8, "masks: %d of %d ", player.masks_collected, map->mask_count);
    mvwprintw(info_win, 3,8, "moves: %d ", MAX_MOVES-player.moves_remaining);
    mvwprintw(info_win, 4,8, "x:%d  y:%d  ",
        map->player[player.player].x, map->player[player.player].y);
    mvwprintw(info_win, 22,4,"m - map toggle");
    wrefresh(info_win);
    #else
    printf("masks: %d of %d ", player.masks_collected, map->mask_count);
    printf("moves remaining: %d ", player.moves_remaining);
    printf("player x:%d  y:%d ",
        map->player[player.player].x, map->player[player.player].y);
    #endif
}
